/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: display.c,v 1.3 2004/08/12 09:42:52 zinnia Exp $
 */

// display management
#include "sps.h"
#include "sps-displaybase.h"
#include "sps-displaytop.h"

static SpsDisplayBase* toplevel;
static SPS_Render_Context* context; // XXX եȥꥹȤФΤ

static GList* zorder;
// static SpsDisplayBase* focus;

void SPS_DisplayManager_ShowTree()
{
	GList* p;
	G_LIST_FOREACH(zorder, p){
		fprintf(stderr, "%s(%s) -> ", sps_displaybase_name(p->data),
			sps_displaybase_visible(p->data) ? "V" : "v");
	}
	fprintf(stderr, "\n");
}
void SPS_DisplayManager_AppendWindow(SpsDisplayBase* _parent, SpsDisplayBase* me)
{
	SpsDisplayBase* parent = _parent;
	if(parent == NULL) parent = toplevel;
	zorder = g_list_append(zorder, me);
	sps_displaybase_appendchild(parent, me);

	SPS_DisplayManager_ShowTree();
}

static void SPS_DisplayManager_RemoveWindow_internal(SpsDisplayBase* me)
{
	zorder = g_list_remove(zorder, me);
}

void SPS_DisplayManager_RemoveWindow(SpsDisplayBase* me)
{
	SPS_DisplayManager_RemoveWindow_internal(me);
	sps_displaybase_removeme(me);
	SPS_DisplayManager_ShowTree();
}

void SPS_DisplayManager_Raise(SpsDisplayBase* me)
{
	zorder = g_list_remove(zorder, me);
	zorder = g_list_append(zorder ,me);
	SPS_DisplayManager_ShowTree();
}

void SPS_DisplayManager_MainLoop()
{
	GList* p;
	SDL_Event e;
	SpsDisplayBase* d;
	g_assert(toplevel != NULL);
	g_assert(zorder != NULL);

	while(toplevel->visible == TRUE){
		SpsDisplayBase* focus = NULL;

		SPS_TextureCount_Reset();

		// search focussed display
		G_LIST_FOREACH(zorder, p){
			d = (SpsDisplayBase*)p->data;
			if(d->visible == TRUE) focus = d;
		}
		while(SDL_PollEvent(&e)){
			if(focus != NULL){
				sps_displaybase_processevent(focus, &e);
			}
		}
#ifdef USE_OPENGL
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		SDL_GL_Enter2DMode(TRUE);
#else
		SDL_FillRect(display->display, NULL, context->setting.backcolor);
#endif

		G_LIST_FOREACH(zorder, p){
			d = (SpsDisplayBase*)(p->data);
			if(d->topmost == FALSE && d->visible == TRUE){
				sps_displaybase_draw(d);
			}
		}
		G_LIST_FOREACH(zorder, p){
			d = (SpsDisplayBase*)(p->data);
			if(d->topmost == TRUE && d->visible == TRUE){
				sps_displaybase_draw(d);
			}
		}

#ifdef USE_OPENGL
		SDL_GL_Leave2DMode();

		SDL_GL_SwapBuffers();
#else
		SDL_UpdateRect(display->display, 0, 0, 0, 0);
#endif

		// cleanup list
		GList* remove_list = NULL;
		G_LIST_FOREACH(zorder, p){
			if(sps_displaybase_isalive(p->data) == FALSE){
				remove_list = g_list_append(remove_list, p->data);
			}
		}
		G_LIST_FOREACH(remove_list, p){
			SpsDisplayBase* d = SPS_DISPLAYBASE(p->data);
			SPS_DisplayManager_RemoveWindow(d);
			g_object_unref(d);
		}
		display->frames ++;
	}
	return;
}

void SPS_DisplayManager_Init(SPS_Render_Context* _ctx)
{
	toplevel = g_object_new(SPS_TYPE_DISPLAYTOP, NULL);
	sps_displaybase_show(toplevel);
	zorder = g_list_append(NULL, toplevel);
	context = _ctx;
}

TTF_Font* SPS_DisplayManager_GetFont(const char* name)
{
	return GetFont(context, name);
}
