/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: flip_iidx7th.c,v 1.6 2004/05/18 03:49:36 zinnia Exp $
 */
#include "sps.h"
#ifdef USE_OPENGL
static const int shade_level = 10;
void draw_with_scale(SPS_Scene* scene, float scalex, float scaley)
{
	if(scene == NULL) return;
	if(scene->page == NULL) return;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glScalef(scalex, scaley, 1.0);
	SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_NONE, NULL, FALSE);

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

static const int ms = 1000;
static void scene1(SPS_Flip_Parameter* param, Uint32 tick_diff)
{
	float ratio = (float)tick_diff / (float)ms;
	float scale = 6.0 - ratio * 5.0; 
	int i;
	glColor4f(1.0, 1.0, 1.0, 1.0);
	SPS_Page_Draw_Bg(param->scene_after->page, 1.0);
	draw_with_scale(param->scene_after, scale, scale);

	for(i = 0; i < shade_level; i++){
		float shadescale = scale + scale * scale * i * 0.05;
		float alpha = 0.9 - (float)i / (float)shade_level * 0.5;
		glColor4f(0.0, 1.0, 1.0, alpha);
		draw_with_scale(param->scene_after, shadescale, shadescale);
	}
}

static void scene2(SPS_Flip_Parameter* param, Uint32 tick_diff)
{
	float ratio = (float)tick_diff / (float)ms;
	float scale = 1.0 + ratio * 5.0;
	int i;

	SPS_Page_Draw_Bg(param->scene_after->page, 1.0);
	for(i = shade_level; i > 0; i--){
		float shadescale  = scale + scale * scale * i * 0.05;
		float alpha = (0.5 - 0.4 * (float)i / (float)shade_level) *
			(1.0 - ratio);

		glColor4f(0.0, 1.0, 1.0, alpha);
		draw_with_scale(param->scene_after, shadescale, 1.0);
	}

	SPS_Draw_Parameter dparam;
	dparam.alpha = 1.0;
	SPS_Scene_Draw_Generic(param->scene_after, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, FALSE);
}
#endif
static gboolean Process(SPS_Flip_Parameter* param)
{
#ifdef USE_OPENGL
	Uint32 tick_diff = param->tick_current - param->tick_begin;
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	if(tick_diff <= ms){
		scene1(param, tick_diff);
	} else if (tick_diff <= ms * 2){
		scene2(param, tick_diff - ms);
	} else {
		scene2(param, ms);
		return FALSE;
	}
	return TRUE;
#else
	return FALSE;
#endif
}

SPS_Flip_Vector iidx7th_vector = {
	TRUE, // require_opengl
	"IIDX7Th", // name
	"Guess what.", // description
	FALSE, // disappear
	TRUE, // appear
	Process,
};
