/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: sps_presentation.c,v 1.11 2004/05/18 03:49:36 zinnia Exp $
 */

#include "sps.h"

static void SPS_Page_Register(SPS_Render_Context* context, GString* text);
static void parse(SPS_Render_Context* context, GString** source, GList* lines);

static void doinclude(SPS_Render_Context* context, GString** dst, GString* line)
{
	// determine file name
	GString* path = g_string_new("");
//	fprintf(stderr, "%s\n", line->str);
	int i;
	gboolean in_quote = FALSE;
	for(i = 0; i < line->len; i ++){
		if(line->str[i] == '"'){
			if(in_quote == TRUE){
				fprintf(stderr, "include '%s'\n", path->str);
				GList* lines = ReadLines(path->str);
				parse(context, dst, lines);
				// XXX free
				return;
			} else {
				in_quote = TRUE;
			}
		} else {
			if(in_quote){
				g_string_append_c(path, line->str[i]);
			}
		}
	}
}

static void parse(SPS_Render_Context* context, GString** source, GList* lines)
{
	GList* p;
	if(lines == NULL) return;

	G_LIST_FOREACH(lines, p){
		GString* line = p->data;
		if(!strncmp(line->str, "sps_include(", 12)){
			doinclude(context, source, line);
		} else {
			g_string_append(*source, line->str);
	
			if(!strncmp(line->str, "sps_endpage()", 13)){
				SPS_Page_Register(context, *source);
				*source = g_string_new("");
			}
		}
	}
	
}
static void read_file(SPS_Render_Context* context, const char* path)
{
	GList* result = ReadLines(path);

	GString* source = g_string_new("");
	parse(context, &source, result);
}

static void SPS_Page_Register(SPS_Render_Context* context, GString* text)
{
	SPS_Page* result = g_malloc0(sizeof(SPS_Page));
	if(result != NULL){
		context->presentation->pages = 
			g_list_append(context->presentation->pages, 
				      result);

		context->presentation->pageno_max ++;
		result->source = text;
//		result->flip_driver = SPS_Flip_Driver_Default();

//		result->bg_sprite = NULL;
	}
}

static SPS_Presentation* SPS_Presentation_new()
{
	SPS_Presentation* result = g_malloc0(sizeof(SPS_Presentation));
	if(result != NULL){
		result->pageno_max = 0;
		result->pages = NULL;
		result->title = NULL;
	}
	return result;
}

static SPS_Render_Context* SPS_Render_Context_new()
{
	SPS_Render_Context* context = g_malloc0(sizeof(SPS_Render_Context));
	if(context != NULL){
		context->lua = lua_open();
		lua_context_prepare(context->lua);
		context->presentation = SPS_Presentation_new();
		SPS_Render_Context_Register(context, context->lua);

		context->pauselevel = 0;
		context->lineno = 0;

		context->sprite_count_max = 0;
		context->line = NULL;

		context->in_linerender = 0;
		context->in_pagerender = 0;

		context->icons = g_hash_table_new(g_str_hash, g_str_equal);
		context->fontpath = g_hash_table_new(g_str_hash, g_str_equal);
		context->fonts = g_hash_table_new(g_str_hash, g_str_equal);

		context->cursor.x = 0;
		context->cursor.y = 0;
		context->setting.bg_style = 0;
		context->setting.bg_sprite = NULL;
		context->setting.bg_alpha = 100;
		context->setting.flip_driver = SPS_Flip_Driver_Default();
		context->setting.halign = SPS_HORIZONTAL_ALIGN_LEFT;
		context->setting.hmargin = 0;
		context->setting.backcolor = SDL_MapRGB(display->format, 255, 255, 255);
		context->setting.forecolor = SDL_MapRGB(display->format, 0, 0, 0);
	}
	return context;
}

gchar* SPS_Presentation_GetTitle(SPS_Presentation* presentation)
{
	gchar* result = presentation->title;
	if(result != NULL) return result;

	if(presentation->pageno_max > 0){
		SPS_Page* p = SPS_Page_Get(presentation, 1);
		result = p->title;
	}
	if(result != NULL) return result;
	return "NONAME";
}

static void SPS_Page_PreRender(SPS_Render_Context* context, SPS_Page* page)
{
	GList* p;
	if(page == NULL) return;
	G_LIST_FOREACH(page->drawunits, p){
		SPS_Sprite* sprite = (SPS_Sprite*)p->data;
		if(sprite == NULL) continue;
		g_queue_push_head(display->queue, sprite);
		context->sprite_count_max ++;
	}
}


void SPS_Page_Parse(SPS_Render_Context* context, SPS_Page* page)
{
	if(page == NULL) return;

	context->page_current = page;
//	fprintf(stderr, "page(SPS_Page_Parse) = %x\n", page);
	lua_dostring(context->lua, page->source->str);

	SPS_Page_PreRender(context, page);
}
void SPS_Page_Parse_All(SPS_Render_Context* context)
{
	GList* p;
	context->pageno = 0;
	G_LIST_FOREACH(context->presentation->pages, p){
		SPS_Page* page = (SPS_Page*)p->data;
		context->pageno ++;
		SPS_Page_Parse(context, page);
	}
}

SPS_Render_Context* SPS_Open_File(const char* path, gboolean parse)
{
	SPS_Render_Context* context = SPS_Render_Context_new();
	if(context != NULL){
		context->path = path;
		read_file(context, SPS_SYSTEM_CONFIG_PATH);

		read_file(context, path);

		if(parse) SPS_Page_Parse_All(context);
	}
	return context;
}

SPS_Page* SPS_Page_Get(SPS_Presentation* presentation, int pageno)
{
	GList* ptr = g_list_nth(presentation->pages, pageno - 1);
	SPS_Page* result = NULL;
	if(ptr != NULL){
		result = (SPS_Page*)ptr->data;
	}
//	return (SPS_Page*)ptr; // $B$R$I$$%P%0$J$N$G;/$7$F$*$3$&(B
	return result;
}

SPS_Page* SPS_Page_GetCurrent(SPS_Presentation* presentation)
{
	return SPS_Page_Get(presentation, display->pageno);
}

static void SPS_SetGLColor(Uint32 color, double alpha)
{
	Uint8 r, g, b;
	SDL_GetRGB(color, display->format, &r, &g, &b);

	glColor4f((float)r / 255.0 , (float)g / 255.0 , (float)b / 255.0, alpha);
}

void SPS_Page_Draw_Bg_Tile(SPS_Page* page, double alpha)
{
	SPS_Sprite* bg = page->setting.bg_sprite;
	int x, y;
	SPS_Draw_Parameter param;

	param.alpha = page->setting.bg_alpha * alpha;
	param.shade = FALSE;

	for(y = -display->h / 2; y < display->h / 2; y += bg->h){
		for(x = -display->w / 2; x < display->w / 2; x += bg->w){
			param.x = x;
			param.y = y;
			SPS_Sprite_Draw_Generic(bg, 
						SPS_DRAW_PARAMETER_ENUM_ALPHA |
						SPS_DRAW_PARAMETER_ENUM_POSITION,
						&param);
		}
	}
}
void SPS_Page_Draw_Bg_Zoom(SPS_Page* page, double alpha)
{
	SPS_Sprite* bg = page->setting.bg_sprite;
	SPS_Sprite_Bind(bg);
#ifdef USE_OPENGL
	if(display->use_opengl){
		glColor4d(1.0, 1.0, 1.0, page->setting.bg_alpha * alpha);
		glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(bg->texcoord_minx, bg->texcoord_miny);
		glVertex2i(-display->w / 2, -display->h / 2);

		glTexCoord2f(bg->texcoord_maxx, bg->texcoord_miny);
		glVertex2i(display->w / 2, -display->h / 2);

		glTexCoord2f(bg->texcoord_minx, bg->texcoord_maxy);
		glVertex2i(-display->w / 2, display->h / 2);

		glTexCoord2f(bg->texcoord_maxx, bg->texcoord_maxy);
		glVertex2i(display->w / 2, display->h / 2);
		glEnd();
	} else
#endif
	{
		
	}
}
void SPS_Page_Draw_Bg_Center(SPS_Page* page, double alpha)
{
	SPS_Sprite* bg = page->setting.bg_sprite;
	SPS_Draw_Parameter param;
	param.alpha = page->setting.bg_alpha * alpha;
	param.shade = FALSE;
	param.x = -bg->w / 2;
	param.y = -bg->h / 2;

	SPS_Sprite_Draw_Generic(bg, 
				SPS_DRAW_PARAMETER_ENUM_ALPHA |
				SPS_DRAW_PARAMETER_ENUM_POSITION,
				&param);
}

void SPS_Page_Draw_Bg(SPS_Page* page, double alpha)
{
// 1. $BEI$j$D$V$7(B
#ifdef USE_OPENGL
	if(display->use_opengl){
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_TRIANGLE_STRIP);
		SPS_SetGLColor(page->setting.backcolor, page->setting.bg_alpha * alpha);
		glVertex2i(-display->w / 2,   -display->h / 2);
		glVertex2i(display->w / 2, -display->h / 2);
		glVertex2i(-display->w / 2,  display->h / 2);
		glVertex2i(display->w / 2, display->h / 2);
		glEnd();
		glEnable(GL_TEXTURE_2D);
	} else
#endif
	{
	}

// 2. BG$BIA2h(B
	if(page->setting.bg_sprite == NULL) return;
	switch(page->setting.bg_style)
	{
	case SPS_BGSTYLE_ZOOM:
		SPS_Page_Draw_Bg_Zoom(page, alpha);
		break;
	case SPS_BGSTYLE_CENTER:
		SPS_Page_Draw_Bg_Center(page, alpha);
		break;
	case SPS_BGSTYLE_TILE:
	default:
		SPS_Page_Draw_Bg_Tile(page, alpha);
		break;
	}
}
static void SPS_Page_Draw_Generic(SPS_Page* page, int pauselevel, SPS_DRAW_PARAMETER_ENUM param_enum, SPS_Draw_Parameter* param, gboolean draw_bg)
{
	GList* p;
	if(page == NULL) return;

#ifdef USE_OPENGL
	if(display->use_opengl){
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	} else 
#endif
	{
	}

	if(draw_bg) SPS_Page_Draw_Bg(page, (param != NULL) ? (param->alpha) : 1.0);

	G_LIST_FOREACH(page->drawunits, p){
		SPS_DrawUnit* u = (SPS_DrawUnit*)p->data;
		if(u == NULL) continue;
		if(u->pauselevel <= pauselevel){
			SPS_Draw_Parameter p;
			p.x = u->sprite.x - display->w / 2;
			p.y = u->sprite.y - display->h / 2;
			if(param != NULL){
				p.alpha = param->alpha;
				p.shade = param->shade;
			}
			SPS_Sprite_Draw_Generic(&(u->sprite), 
						param_enum | 
						SPS_DRAW_PARAMETER_ENUM_POSITION,
						&p);
		}
	}
}

void SPS_Scene_Draw_Generic(SPS_Scene* scene, SPS_DRAW_PARAMETER_ENUM param_enum, SPS_Draw_Parameter* param, gboolean draw_bg)
{
	SPS_Page_Draw_Generic(scene->page, scene->pauselevel, param_enum, param, draw_bg);
}

gchar* get_euc_string(const char* str)
{
	gchar* result = g_convert(str, strlen(str),  DEFAULT_ENCODING, "UTF-8",
			 NULL, NULL, NULL);
	return result;
}


void SPS_Page_Dump(SPS_Page* page)
{
	if(page == NULL) return;

	GList* p;
	G_LIST_FOREACH(page->drawunits, p){
		char* t;
		SPS_DrawUnit* u = (SPS_DrawUnit*)p->data;
		if(u == NULL) continue;

		t = "Unknown";
		switch(u->type){
		case SPS_DRAWUNIT_TYPE_PRIMITIVE:
			t = "Primitive";
			break;
		case SPS_DRAWUNIT_TYPE_LETTER:
			t = get_euc_string(u->ch);
			break;
		}
		fprintf(stderr, "%d: %s\n", u->lineno, t);
	}
}
