/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: flip_blur.c,v 1.3 2004/05/18 03:49:36 zinnia Exp $
 */
#include "sps.h"

static const int level_max = 11;

static gboolean Process(SPS_Flip_Parameter* param)
{
	const int ms = 500;
	int i;
	SPS_Scene* scene = param->scene_after;
	double ratio = (double)(param->tick_current - param->tick_begin) / (double)ms;
	if(scene == NULL) return FALSE;
	if(ratio > 1.0) ratio = 1.0;


	double alpha_base = 0.2 + ratio * 0.8;
	SPS_Draw_Parameter dparam;
	dparam.alpha = alpha_base;
	SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, TRUE);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	for(i = 1; i <= level_max; i += 3)
	{
//		double alpha = 0.5 - ((double)(level_max - i) / (double)level_max) * 0.5;
		double alpha = 0.2 * (1.0 - ratio);

		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glTranslatef(-i, 0, 0);
		dparam.alpha = alpha;
		SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, TRUE);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(i, 0, 0);
		SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, TRUE);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(0, -i, 0);
		SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, TRUE);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(0, i, 0);
		SPS_Scene_Draw_Generic(scene, SPS_DRAW_PARAMETER_ENUM_ALPHA, &dparam, TRUE);
		glPopMatrix();

	}

	if(ratio == 1.0) return FALSE;
	return TRUE;
}

SPS_Flip_Vector blur_vector = {
	TRUE, // require_opengl
	"Blur", // name
	"blur", // description
	FALSE, // disappear
	TRUE, // appear
	Process,
};

	
