/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: main.c,v 1.18 2004/05/18 03:49:36 zinnia Exp $
 */

#include "sps.h"

#include "sps-displaybase.h"
#include "sps-displaypresentation.h"
#include "sps-displayloading.h"

SPS_Display* display;

SPS_Flip_Parameter* SPS_Flip_Parameter_new(
	SPS_Presentation* presentation,
	SPS_Scene* scene_before,
	SPS_Scene* scene_after)
{
	SPS_Flip_Parameter* result = g_malloc0(sizeof(SPS_Flip_Parameter));
	if(result != NULL){
		result->tick_begin = SDL_GetTicks();
		result->tick_current = result->tick_begin;
		result->userdata = NULL;
		result->scene_before = scene_before;
		result->scene_after = scene_after;
	}
	return result;
}

static void SPS_FPS_Reset()
{
	display->tick_begin = SDL_GetTicks();
	display->frames = 0;
	display->fps = 0;
}

static void SPS_TextureCount_Reset()
{
	display->textures = 0;
}

static void showfps(SPS_Presentation* presentation)
{
	static char buf[0x100];
	static SPS_Sprite* tmp = NULL; // XXX should be in SDL_Display
	SDL_Color color;
	int fps;
	Uint32 tick_current;
	Uint32 tick_diff;

	if(tmp == NULL){
		tmp = SPS_Sprite_new_WithSize(260, KANJI_FONT_SIZE);
	}
	SDL_FillRect(tmp->surface, NULL, SDL_MapRGB(tmp->surface->format,
						    0, 0, 0));


	tick_current = SDL_GetTicks();

	tick_diff = tick_current - display->tick_begin;
	if(tick_diff < 800) {
		fps = display->fps;
	} else {
		fps = display->frames * 1000 / tick_diff;
		SPS_FPS_Reset();
		display->fps = fps;
	}

	color.r = color.g = 255;
	color.b = 0;

	sprintf(buf, "Page: %d/%d, %d textures, %d fps", display->pageno, 
		presentation->pageno_max, 
		display->textures,
		fps);
	Kanji_PutText(display->kanji_font, 0, 0, tmp->surface, buf, color);

	tmp->surface_dirty = TRUE;
	tmp->x = display->w - tmp->surface->w;
	tmp->y = display->h - tmp->surface->h;
	tmp->w = tmp->surface->w;
	tmp->h = tmp->surface->h;

/*	glColor4f(1.0, 1.0, 1.0, 1.0);
	SPS_Sprite_Bind(tmp);
	SPS_Sprite_Draw_WithPosition(tmp, tmp->x, tmp->y);*/
	SPS_Draw_Parameter p;
	p.x = tmp->x;
	p.y = tmp->y;
	p.alpha = 1.0;
	p.shade = FALSE;
	SPS_Sprite_Draw_Generic(tmp, 
				SPS_DRAW_PARAMETER_ENUM_POSITION |
				SPS_DRAW_PARAMETER_ENUM_ALPHA,
				&p);

}

// XXX
SPS_Display* SPS_Display_new()
{
	return g_malloc0(sizeof(SPS_Display));
}

/* XXX not called */
static void destroy()
{
	/* XXX release contents */
	
#if 0
	g_hash_table_destroy(context->icons);
	g_hash_table_destroy(context->fontpath);
	g_hash_table_destroy(context->font);

	SDL_FreeSurface(context->surface);
#endif

}

static void parse_geometry(const char* expression, int* w, int* h)
{
	sscanf(expression, "%dx%d", w, h);
	return;
}

//#define MAXFPS (60)
//#define DISPLAY_DELAY (1000 / (MAXFPS))

// $B%U%!%$%k%Q!<%9MQ%9%l%C%I(B
static int parsing_thread(void* data)
{
	SDL_Event e;
	SPS_Render_Context* context = data;
	SPS_Page_Parse_All(context);
//	while(!g_queue_is_empty(display->queue)){
//		SDL_Delay(100);
//	}
	e.type = SDL_USEREVENT;
	e.user.code = SPS_USEREVENT_PARSE_DONE;

	// $BJL%9%l%C%I$+$i%$%Y%s%H$r(BPush$B$9$k$N$O$^$:$$$s$@$m$&$J$"(B
	SDL_PushEvent(&e);
	return 0;
}

#if 0

// $B!V%U%!%$%kFI$_9~$_Cf!W$N=hM}(B
// SPS_STATE_PARSE: $B%Q!<%9$N?JD=$rI=<((B
// SPS_STATE_READY: PRESS ANY KEY TO START     
static void loading_loop(SPS_Render_Context* context)
{
	SPS_Sprite* message;
	SPS_Sprite* texture_message;
	static char buf[0x100];

	SDL_Color color;
	message = SPS_Sprite_new_WithSize(200, KANJI_FONT_SIZE);
	texture_message = SPS_Sprite_new_WithSize(200, KANJI_FONT_SIZE);


	color.r = color.g = 255;
	color.b = 0;

	SDL_Event e;
	while(1){
		if(SDL_PollEvent(&e)){
			switch(e.type){
			case SDL_QUIT:
				exit(1);
				break;
			case SDL_KEYDOWN:
				if(SPS_State_Get() == SPS_STATE_READY){
					return; // exit loading loop
				}
				break;
			}
		}
		// $B$,$a$s$N$3$&$7$s(B
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		SDL_GL_Enter2DMode(FALSE);
		SDL_FillRect(message->surface, NULL, SDL_MapRGB(message->surface->format, 0, 0, 0));
		SDL_FillRect(texture_message->surface, NULL, SDL_MapRGB(texture_message->surface->format, 0, 0, 0));
		
		switch(SPS_State_Get()){
		case SPS_STATE_PARSE:
			sprintf(buf, "Loading %d/%d ...", 
				context->pageno,
				context->presentation->pageno_max);
			Kanji_PutText(display->kanji_font, 0, 0, 
				      message->surface, buf, color);
			sprintf(buf, "PreBinding sprite: %d/%d",
				display->sprite_count,
				context->sprite_count_max
				);
			Kanji_PutText(display->kanji_font, 0, 0,
				      texture_message->surface, buf, color);
			break;
		case SPS_STATE_READY:
			Kanji_PutText(display->kanji_font, 0, 0,
				      message->surface, 
				      "Press any key to start...",
				      color);
			break;
		default:
			// this should not be happend.
			break;
		}
		message->surface_dirty = TRUE;
		message->x = (display->w - message->surface->w) / 2;
		message->y = (display->h - message->surface->h) / 2;
		message->w = message->surface->w;
		message->h = message->surface->h;

/*		glColor4f(1.0, 1.0, 1.0, 1.0);
		SPS_Sprite_Bind(message);
		SPS_Sprite_Draw_WithPosition(message, message->x, message->y);*/
		SPS_Draw_Parameter p;
		p.x = message->x;
		p.y = message->y;
		p.alpha = 1.0;
		p.shade = FALSE;
		SPS_Sprite_Draw_Generic(message, 
					SPS_DRAW_PARAMETER_ENUM_ALPHA |
					SPS_DRAW_PARAMETER_ENUM_POSITION,
					&p);

		if(SPS_State_Get() == SPS_STATE_PARSE){
			texture_message->surface_dirty = TRUE;
			texture_message->x = (display->w - texture_message->surface->w) / 2;
			texture_message->y = (display->w - texture_message->surface->w) / 2 + KANJI_FONT_SIZE * 2;
			texture_message->w = texture_message->surface->w;
			texture_message->h = texture_message->surface->h;
/*			SPS_Sprite_Bind(texture_message);
			SPS_Sprite_Draw_WithPosition(texture_message,
						     texture_message->x,
						     texture_message->y);*/
			SPS_Draw_Parameter p;
			p.x = texture_message->x;
			p.y = texture_message->y;
			p.alpha = 1.0;
			p.shade = FALSE;
			SPS_Sprite_Draw_Generic(texture_message,
						SPS_DRAW_PARAMETER_ENUM_ALPHA |
						SPS_DRAW_PARAMETER_ENUM_POSITION,
						&p);
						     
		}

		SDL_GL_Leave2DMode();
		SDL_GL_SwapBuffers();

		Uint32 tick_start = SDL_GetTicks();
		Uint32 tick_diff;
		while(!g_queue_is_empty(display->queue)){
			tick_diff = SDL_GetTicks() - tick_start;
			if(tick_diff > 100) break;
			SPS_Sprite* s = g_queue_pop_tail(display->queue);
			SPS_Sprite_Bind(s);
			display->sprite_count ++;
		}
	}
}
#endif

int main(int argc, char* argv[])
{
	int i;

	char* path = NULL;
	SPS_Render_Context* context;
	display = SPS_Display_new();
	
	display->fullscreen = 0;
	display->w = 800;
	display->h = 600;

	display->sprite_shade = TRUE;
	display->showfps = 1;

	for(i = 1; i < argc; i++){
		switch(*argv[i]){
		case '-':
			switch(*(argv[i] + 1)){
			case 'h':
			case '?':
				usage();
				break;

			case 'g':
				i++;
				parse_geometry(argv[i],
					       &(display->w),
					       &(display->h));
				break;
			case 'f':
				display->fullscreen = 1 - display->fullscreen;
				break;
			case 'F':
				display->showfps = 0;
				break;
			}
			break;
		default:
			path = argv[i];
			break;
		}
	}
	if(path == NULL) usage();

	g_type_init();

	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		g_error("SDL_Init: %s", SDL_GetError());
		return 0;
	}
	if(TTF_Init() < 0){
		g_error("TTF_Init: %s", SDL_GetError());
		return 0;
	}

#ifdef USE_OPENGL
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
#endif
	atexit(SDL_Quit);


#ifdef USE_OPENGL
#define ADDITIONAL_FLAGS SDL_OPENGL
#else
#define ADDITIONAL_FLAGS SDL_SWSURFACE
#endif

	display->display = 
		SDL_SetVideoMode(display->w, 
				 display->h, 
				 32,
				 ADDITIONAL_FLAGS | 
				 (display->fullscreen ? SDL_FULLSCREEN : 0));

	if(display->display == NULL){
		g_error("SDL_SetVideoMode: %s", SDL_GetError());
		return 0;
	}


	SPS_FPS_Reset();

	display->format = display->display->format; // XXX
	display->pageno = 1;

#ifdef USE_OPENGL
	setup_opengl();
#endif

	display->kanji_font = Kanji_OpenFont(KANJI_FONT_FILENAME, KANJI_FONT_SIZE);
	if(display->kanji_font == NULL){
		g_error("Kanji_OpenFont: error.");
	}

	context = SPS_Open_File(path, FALSE);
	g_debug("%d pages.", context->presentation->pageno_max);

	if(context->presentation->pageno_max <= 0){
		fprintf(stderr, "%s: No pages to show.\n", path);
		return 1;
	}

	display->queue = g_queue_new();
	display->sprite_count = 0;

	context->thread = SDL_CreateThread(parsing_thread, (void*)context);
  
	display->tick_begin = SDL_GetTicks();
	display->frames = 0;

	SPS_DisplayManager_Init();

	SpsDisplayLoading* l = g_object_new(SPS_TYPE_DISPLAYLOADING, NULL);
	sps_displayloading_set_render_context(l, context);

	SPS_DisplayManager_AppendWindow(NULL, l);
	sps_displaybase_show(SPS_DISPLAYBASE(l));

	SPS_DisplayManager_MainLoop();
	return 0;
}
